#!/usr/bin/env python
import sys,os,re,string

var_pat = re.compile(r'\$\(([^\)]+)\)')
fcmp_pat = re.compile(r'^FCOMPL=(.+)$')

def get_libs():
   '''Try to figure out that libraries we're going to need.  This is
   a very tricky business and this routine is mostly based on
   trial-and-error when trying to compile on different systems.'''

   pglibs = []
   # Here's the logic:  - we take PGPLOT_LIBS as gospel if it is set
   #                    - next, if $PGPLOT_DIR/makefile exists, 
   #                      look at how cpgdemo was built and use those LIBS
   #                    - if all else fails, as the user to set 
   #                      PGPLOT_LIBS
   if "PGPLOT_LIBS" in os.environ:
      pglibs = os.environ['PGPLOT_LIBS']
      print "*********** Using PGPLOT_LIBS environment variable *************"
      return(pglibs)
   elif "PGPLOT_DIR" in os.environ:
      if os.path.exists(os.path.join(os.environ['PGPLOT_DIR'],"makefile")):
         pgmake = open(os.path.join(os.environ['PGPLOT_DIR'], "makefile"))
         lines = pgmake.readlines()
         pgmake.close()
         for i in range(len(lines)):
            if re.search('-o cpgdemo', lines[i]):
               # Found the cpgdemo line.  Extract variables that are used.
               vars = var_pat.findall(lines[i])
         # Look for all variables but the first (the linker)
         pglibs = []
         print "VARIABLES = ", vars[1:]
         for var in vars[1:]:
            for line in lines:
               res = re.search(r'^%s=(.+)' % var, line)
               if res:
                  # found the variable definition
                  libs = res.group(1)
                  # remove the pwd command:
                  libs = libs.replace("-L`pwd`","")
                  print "VAR",var," has value",libs
                  pglibs.append(libs)
      else:
         print "******** $PGPLOT_DIR/makefile not found *************"
         print "You're going to have to set PGPLOT_LIBS by hand"
         return(None)
   else:
      print "************* PGPLOT_DIR envrionment variable not set ************"
      print "Please set PGPLOT_DIR or set PGPLOT_LIBS"
      return(None)
   # Now, we check for the version of the fortran compiler used to compile
   # libpgplot
   fcompiler = 'unknown'      # assume this
   for line in lines:
      res = fcmp_pat.search(line)
      if res:
         fcompiler = res.group(1)

   if fcompiler == 'unknown':  
      print "******* PGLPOT was compiled with unrecognized FORTRAN compiler***"
      print " Forging ahead with what we've got, but you might need to set"
      print " PGPLOT_LIBS by hand"
      return(string.join(pglibs))

   # Okay, NOW we have to figure out where libg2c or libgfortran is:
   f = os.popen('%s -print-search-dirs' % fcompiler)
   lines = f.readlines()
   lines = map(string.strip, lines)
   f.close()
   lib_dir = None
   ext_lib = {'g77':'libg2c.a', 'gfortran':'libgfortran.a'}
   for line in lines:
      if line.find('libraries:') == 0:
         line = line.replace('libraries: =','')
         dirs = line.split(':')
         for dir in dirs:
            if os.path.isdir(dir):
               if os.path.isfile(os.path.join(dir,ext_lib[fcompiler])):
                  lib_dir = dir
                  break
   if lib_dir is None:
      print "************ Warning:  %s not found *********" % ext_lib[fcompiler]
      print "%s not found in any of the default search-paths.  You may need" \
            % (ext_lib[fcompiler])
      print "to install development files or set PGPLOT_LIBS by hand."
      
   else:
      pglibs.append('-L%s -l%s' % (lib_dir, \
            {'g77':'g2c','gfortran':'gfortran'}[fcompiler]))
   return(string.join(pglibs))

def configuration(parent_package='', top_path=None):
   from numpy.distutils.misc_util import Configuration

   pglibs = get_libs()
   print "PGLIBS:"
   print pglibs
   # First, find what extra libs we need
   if pglibs is None: 
      sys.exit(1)

   pglibs = pglibs.split()
   pglibdirs = []
   pglibraries = []
   for w in pglibs:
      if w[:2] == "-L":  pglibdirs.append(w[2:])
      elif w[:2] == "-l":  pglibraries.append(w[2:])
      elif w[:4] == "-Wl,":  continue
      else:  pglibraries.append(w)

   library_dirs = pglibdirs
   if 'PGPLOT_DIR' in os.environ:
      library_dirs.append(os.environ['PGPLOT_DIR'])
   for dir in ['PYTHONBASE','SCIDIR','SCISOFTDIR']:
      if dir in os.environ:
         library_dirs.append(os.environ[dir])

   config = Configuration('pygplot', parent_package, top_path)
   config.add_extension('cpgplot',
                       sources = ["cpgplot/cpgplot_wrap.c"],
                       include_dirs = ['./cpgplot'],
                       libraries = ['cpgplot','pgplot'] + pglibraries,
                       library_dirs = library_dirs)
   return config

if __name__ == '__main__':
   try:
      from numpy.distutils.core import setup
   except ImportError:
      print "Sorry, you need Numpy (Numerical Python) installed before you"
      print "can use pygplot.  Get it from http://numpy.scipy.org"
      sys.exit(1)

   setup(version="0.97",
      description="An Object Oriented wrapper around the PGPLOT Library",
      author="Chris Burns & Dan Kelson",
      author_email="cburns@obs.carnegiescience.edu",
      url="http://www.ociw.edu/Code/python/burns-python-scripts/pygplot/",
      **configuration(top_path='').todict())

